#include "GenAlg.h"
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <random>
#include "OptScheduling.h"
#include "FirstOptScheduling.h"

GenAlg::GenAlg(GenType genType):mGenType(genType)
{
    mOptScheduling = std::make_unique<FirstOptScheduling>();
}

double GenAlg::random(int min , int max)
{
    double m1 = (double)(rand() % 101) / 101;                        //  01֮С,õֵΪ(0,1)
    min++;                                                                             // Ϊ(min+1,max),
    double m2 = (double)((rand() % (max - min + 1)) + min);    // min+1,max ֮õֵΪ[min+1,max]
    m2 = m2 - 1;                                                                        //ֵΪ[min,max-1]
    return m1 + m2;                                                                //ֵΪ(min,max),Ϊ
}

void GenAlg::init(int popsize, double MutRate, double CrossRate, int GenLength, double LeftPoint, double RightPoint)
{
    mPopSize = popsize;
    mMutationRate = MutRate;
    mCrossoverRate = CrossRate;
    mChromoLength = GenLength;
    mTotalFitness = 0;
    mGeneration = 0;

    mBestFitness = 0.0;
    mWorstFitness = 99999999;
    mAverageFitness = 0;
    mMaxPerturbation = 0.001;

    mLeftPoint = LeftPoint;
    mRightPoint = RightPoint;

    // ȺԳʼ  
    vecPop.clear();
    for (int i = 0; i < mPopSize; i++)
    {
        // Ĺ캯ѾӦֳʼΪ0  
        vecPop.push_back(Genome());

        // еĻʼΪڵ  
        for (int j = 0; j < mChromoLength; j++)
        {
            switch (mGenType)
            {
            case RealValueCoding:
                vecPop[i].vecGenome.push_back(random(0,MAX_RANDOM) * (mRightPoint - mLeftPoint) + mLeftPoint);
                break;
            case BinaryCoding:
                vecPop[i].mBinaryGenVec.push_back((int)random() % 2);
                break;
            default:
                break;
            }
            
        }
    }
}

void GenAlg::mutate(vector<double>& chromo)
{
    // ѭԤͻ,Իͻ  
    for (int i = 0; i < chromo.size(); ++i)
    {
       // ͻĻ  
        if (random(0,1) < mMutationRate)
        {
            // ʹȨֵӻ߼һСֵ  
            chromo[i] += ((random() - 0.5) * mMaxPerturbation);

            // ұ߽  
            if (chromo[i] < mLeftPoint)
            {
                chromo[i] = mRightPoint;
            }
            else if (chromo[i] > mRightPoint)
            {
                chromo[i] = mLeftPoint;
            }
        }
    }
}

void GenAlg::mutate(vector<char>& chromo)
{
    // ѭԤͻ,Իͻ  
    for (int i = 0; i < chromo.size(); ++i)
    {
        // ͻĻ  
        if (random(0,1) < mMutationRate)
        {
            chromo[i] =( chromo[i]&0x01) ^ 0x01;
        }
    }
}

void GenAlg::select()
{

}

void GenAlg::corssver()
{
    // Ⱦɫ
    int popSize = vecPop.size();

    // Ⱥ
    std::vector<int> randIndexVec;
    for (int i = 0; i < popSize; i++)
        randIndexVec.push_back(i);

    // ΪȾɫ
    std::random_shuffle(randIndexVec.begin(), randIndexVec.end());

    int left = 0;
    int right = popSize-1;
    while (left < right)
    {
        int indexLeft = randIndexVec[left];
        int indexRight = randIndexVec[right];

        // Ⱦɫ廥
        crossover(vecPop[indexLeft].mBinaryGenVec, vecPop[indexRight].mBinaryGenVec);

        left++;
        right--;
    }
}

void GenAlg::crossover(vector<char>& chromo1, vector<char>& chromo2)
{
    if (chromo1.size() != chromo2.size())
        return;

    int total = chromo1.size();
   
    // 򽻲
    for (int i = 0; i < total; i++)
    {
        // 㽻
        if (random(0, 1) < mCrossoverRate)
        {
            int index = random(0, total);
            for (int i = index; i < total; i++)
            {
                swap(chromo1[i], chromo2[i]);
            }          
        }    
    }
}

Genome GenAlg::chromoRoulette()
{
    // һ 0 ȺӦ֮ܺ.  mTotalFitness¼ȺӦԷܺ
    double slice = (random(0,1)) * mTotalFitness;

    // 򽫳תѡǸ.  
    Genome theChosenOne;

    // ۼӦԷĺ.  
    double fitnessSoFar = 0;

    // ȺÿһȾɫ塣  
    for (int i = 0; i < mPopSize; ++i)
    {
        //ۼӦԷ.  
        fitnessSoFar += vecPop[i].fitness;

        // ۼƷ,ѡʱĻ.  
        if (fitnessSoFar >= slice)
        {
            theChosenOne = vecPop[i];
            break;
        }
    }
    //תѡĸ  
    return theChosenOne;
}

void GenAlg::calculateBestWorstAvTot()
{
    int total = vecPop.size();
    for (int i = 0; i < total; i++)
    {
        double fitNessTemp = vecPop[i].fitness;

        // Ӧֵ
        mTotalFitness += fitNessTemp;

        // Ӧֵ
        if (mBestFitness < fitNessTemp)
            mBestFitness = fitNessTemp;

        // Ӧֵ
        if (mWorstFitness > fitNessTemp)
            mWorstFitness = fitNessTemp;
    }

    // ƽӦ
    mAverageFitness = mTotalFitness / (double)total;
}

void GenAlg::reset()
{
    mPopSize = 0;
    mMutationRate = 0;
    mCrossoverRate =0;
    mChromoLength = 0;
    mTotalFitness = 0;
    mGeneration = 0;

    mBestFitness = 0.0;
    mWorstFitness = 99999999;
    mAverageFitness = 0;
    mMaxPerturbation = 0.001;

    mLeftPoint =0;
    mRightPoint = 0;

    // ȺԳʼ  
    vecPop.clear();
}

void GenAlg::epoch(vector<Genome>& vecNewPop)
{

}

Genome GenAlg::bestFitness()
{
    int maxIndex = -1;
    for (int i = 0; i < vecPop.size(); i++)
    {
        if (mBestFitness == vecPop[i].fitness)  
            maxIndex = i;    
    }

    // ûҵһյĻͶ
    if (maxIndex == -1)
        return Genome();

    return vecPop[maxIndex];
}

double GenAlg::averageFitness()
{
    return mAverageFitness;
}

double  GenAlg::fitnessfunction(float cost, map<string, double>& monitorValues)
{
    // Ӧ
    double objVal = mOptScheduling->objectiveFunction(cost, monitorValues);
    return objVal;
}

void GenAlg::encoding(map<string, int>& phenotype, vector<char>& chromo)
{
    map<string, int>::iterator iter = phenotype.begin();
    for (; iter != phenotype.end(); iter++)
    {

    }
}

void GenAlg::decoding(vector<map<string, int>>& phenotype)
{
    size_t total = vecPop.size();
    
    for (int i = 0; i < total; i++)
    {
        Genome genome = vecPop[i];
        vector<char> genVec = genome.mBinaryGenVec;
        for (int j = 0; j < genVec.size(); j++)
        {
            phenotype[i].insert(pair<string, char>(mPumpBits[j], genVec[j]));
        }     
    }
}

void GenAlg::decodingSingle(vector<char>& chromo, map<string, int>& phenotype)
{
    for (int j = 0; j < chromo.size(); j++)
    {
        phenotype.insert(pair<string, char>(mPumpBits[j], chromo[j]));
    }
}

void GenAlg::setMaxMonitorVals(const  map<string, double>& maxMonitors)
{
    mOptScheduling->setMaxMonitorVals(maxMonitors);
}

void GenAlg::setMinMonitorsVals(const map<string, double>& minMonitors)
{
    mOptScheduling->setMinMonitorsVals(minMonitors);
}

void GenAlg::setPumpBit(const vector<string>& pumpBit)
{
    mPumpBits = pumpBit;
}