#include "CivOptSchedEngine.h"
#include "CivInpConvertor.h"
#include "GenAlg.h"
#include "FirstOptScheduling.h"
#include "CivPumpHelper.h"
#include "CivSchedulingCompute.h"
#include "Genome.h"



CivOptSchedEngine::CivOptSchedEngine(const string& uri):mUri(uri)
{
	mMutRate = 0.45;
	mCrossRate = 0.07;
}

bool CivOptSchedEngine::optimalScheduling()
{
	// תinpļ
	CivInpConvertor convertor(mUri);
	string inpFile = convertor.convertBaseInp();

	// ȡˮø,ˮøȷ볤
	CivPumpHelper pumpHelper(mUri);
	mGenLength = pumpHelper.getPumpNumbers();

	// ȡźcodeӳ
	map<string, string> snToCodeMap;
	pumpHelper.getMap("", "code", snToCodeMap);

	// ȡż
	vector<string> snVec;
	pumpHelper.getPumpSn(snVec);

	// ʼŴ㷨
	GenAlg genAlg(GenAlg::BinaryCoding);

	// ˮõıλ
	genAlg.setPumpBit(snVec);

	// Ⱥʼ
	popsize = 100;
	genAlg.init(popsize, mMutRate, mCrossRate, mGenLength, 0, 0);
	
	// ˮ
	CivSchedulingCompute schedulingCompute;
	vector<string> monitorVec;
	for (auto iter = mMonitoring.begin(); iter != mMonitoring.end(); iter++)
	{
		monitorVec.push_back(iter->first);
	}
	schedulingCompute.setMonitors(monitorVec);
	schedulingCompute.openFile(inpFile);

	// ԭʼһˮǰˮ
	if (!schedulingCompute.calculate())
		return false;

	// ԭʼģֵ
	schedulingCompute.getMonitorsValue(mBforeSchedulingResults);

	// ʼ
	int index = 0;
	while (index++ < popsize)
	{
		// ƽ
		vector<map<string, int>>monitorsMap;
		genAlg.decoding(monitorsMap);

		// ʶ

		// ݸӦȽѡ

		// Ӧ

		// һȺ
	}

	// ȡõ
	Genome genome = genAlg.bestFitness();
	vector<char> genVec = genome.mBinaryGenVec;

	// ˮ
	genAlg.decodingSingle(genVec, mPumpBestStatus);

	// һˮ㣬Ⱥֵ
	schedulingCompute.updatePumpStatus(mPumpBestStatus);
	schedulingCompute.calculate();

	// еֵ
	schedulingCompute.getMonitorsValue(mBforeSchedulingResults);

	// 
	schedulingCompute.close();
	return true;
}

void CivOptSchedEngine::schedulingResultToJson(string& json)
{
	// ˮ:
	json.append("{\"ˮ״̬\":{");

	map<string, int>::iterator iter = mPumpBestStatus.begin();
	for (;iter!=mPumpBestStatus.end();iter++)
	{
		json.append(iter->first + ":" + to_string(iter->second));
		json.append(",");
	}
	json = json.substr(0, json.length() - 1);
	json.append("},");

	// 
	json.append("\":[\"");
	int total = mBforeSchedulingResults.size();
	for (int i = 0; i < total; i++)
	{
		map<string, double> beforeResult = mBforeSchedulingResults[i];
		map<string, double> afterResult = mAfterSchedulingResults[i];

		json.append("{");
		for (auto iter = beforeResult.begin(); iter != beforeResult.end(); iter++)
		{
			string monitorName = iter->first;
			double beforeMonitorVal = iter->second;

			auto afteriter = afterResult.find(monitorName);
			if (afteriter == afterResult.end())
				continue;

			double afterMonitorvVal = afteriter->second;

			json.append("\""+monitorName+"\":[");
			json.append(to_string(beforeMonitorVal));
			json.append(",");
			json.append(to_string(afterMonitorvVal));
			json.append("],");
		}
		json = json.substr(0, json.length() - 1);
		json.append("},");
	}
	json.substr(0, json.length() - 1);
	json.append("]}");
}

void CivOptSchedEngine::setMonitor(const map<string, vector<double>>& monitorMap)
{
	mMonitoring = monitorMap;
}

void CivOptSchedEngine::setDecisionVariableStatusBit(const map<int,string>& statusMap)
{
	mStatusBit = statusMap;
}

void CivOptSchedEngine::setStartTime(const string& time)
{
	mStartTime = time;
}

void CivOptSchedEngine::setEndTime(const string& time)
{
	mEndTime = time;
}